@@ -16,7 +16,7 @@ from result import Err, Ok, Result
 
 from freewili.framing import ResponseFrame
 from freewili.fw_serial import FreeWiliSerial
-from freewili.types import ButtonColor, EventType, FreeWiliProcessorType, IOMenuCommand
+from freewili.types import ButtonColor, EventType, FileSystemContents, FreeWiliProcessorType, IOMenuCommand
 
 # USB Locations:
 # first address = FTDI
@@ -926,6 +926,33 @@ class FreeWili:
             case _:
                 raise RuntimeError("Missing case statement")
 
+    def enable_audio_events(
+        self,
+        enable: bool,
+        processor: FreeWiliProcessorType = FreeWiliProcessorType.Display,
+    ) -> Result[str, str]:
+        """Enable or disable audio events.
+
+        Arguments:
+        ----------
+            enable: bool
+                Whether to enable or disable audio events.
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        ---------
+            Result[ResponseFrame, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.enable_audio_events(enable)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
     def process_events(self) -> None:
         """Process any events that have been received.
 
@@ -1063,7 +1090,7 @@ class FreeWili:
                 raise RuntimeError("Missing case statement")
 
     def write_uart(
-        self, data: bytes, processor: FreeWiliProcessorType = FreeWiliProcessorType.Main
+        self, data: bytes | str, processor: FreeWiliProcessorType = FreeWiliProcessorType.Main
     ) -> Result[str, str]:
         """Write uart data.
 
@@ -1087,6 +1114,329 @@ class FreeWili:
             case _:
                 raise RuntimeError("Missing case statement")
 
+    def send_ir(
+        self, data: bytes, processor: FreeWiliProcessorType = FreeWiliProcessorType.Display
+    ) -> Result[str, str]:
+        """Send IR data.
+
+        Notes: v54 firmware uses NEC format and is converted to an 32-bit integer.
+
+        Parameters:
+        ----------
+            data : bytes
+                The data to send. The first 4 bytes are used as the command.
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        -------
+            Result[bytes, str]:
+                Ok(bytes) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.send_ir(data)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def play_audio_file(
+        self, file_name: str, processor: FreeWiliProcessorType = FreeWiliProcessorType.Display
+    ) -> Result[str, str]:
+        """Play an audio file on the FreeWili.
+
+        Arguments:
+        ----------
+            file_name: str
+                Name of the file in the FreeWili. 8.3 filename limit exists as of V12
+            processor: FreeWiliProcessorType
+                Processor to upload the file to.
+
+        Returns:
+        -------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.play_audio_file(file_name)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def play_audio_asset(
+        self, asset_value: str | int, processor: FreeWiliProcessorType = FreeWiliProcessorType.Display
+    ) -> Result[str, str]:
+        """Play an audio asset on the FreeWili.
+
+        Arguments:
+        ----------
+            asset_value: str | int
+                The asset value to play.
+            processor: FreeWiliProcessorType
+                Processor to upload the file to.
+
+        Returns:
+        -------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.play_audio_asset(asset_value)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def play_audio_number_as_speech(
+        self, value: int, processor: FreeWiliProcessorType = FreeWiliProcessorType.Display
+    ) -> Result[str, str]:
+        """Play an audio number as speech on the FreeWili.
+
+        Arguments:
+        ----------
+            value: int
+                The audio number to play.
+            processor: FreeWiliProcessorType
+                Processor to upload the file to.
+
+        Returns:
+        -------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.play_audio_number_as_speech(value)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def play_audio_tone(
+        self,
+        frequency_hz: int,
+        duration_sec: float,
+        amplitude: float,
+        processor: FreeWiliProcessorType = FreeWiliProcessorType.Display,
+    ) -> Result[str, str]:
+        """Play an audio tone on the FreeWili.
+
+        Arguments:
+        ----------
+            frequency_hz: int
+                The frequency of the tone in Hertz.
+            duration_sec: float
+                The duration of the tone in seconds.
+            amplitude: float
+                The amplitude of the tone (0.0 to 1.0).
+            processor: FreeWiliProcessorType
+                Processor to upload the file to.
+
+        Returns:
+        -------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.play_audio_tone(frequency_hz, duration_sec, amplitude)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def record_audio(
+        self, file_name: str, processor: FreeWiliProcessorType = FreeWiliProcessorType.Display
+    ) -> Result[str, str]:
+        """Record audio on the FreeWili.
+
+        Arguments:
+        ----------
+            file_name: str
+                Name of the file in the FreeWili. (ie. "/sounds/test.wav")
+            processor: FreeWiliProcessorType
+                Processor to upload the file to.
+
+        Returns:
+        -------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.record_audio(file_name)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def change_directory(self, directory: str, processor: FreeWiliProcessorType) -> Result[str, str]:
+        """Change the current directory for file operations.
+
+        Arguments:
+        ----------
+            directory: str
+                The directory to change to.
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        ---------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.change_directory(directory)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def create_directory(self, directory: str, processor: FreeWiliProcessorType) -> Result[str, str]:
+        """Create a new directory on the FreeWili.
+
+        Arguments:
+        ----------
+            directory: str
+                The directory to create.
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        ---------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.create_directory(directory)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def remove_directory_or_file(self, dir_or_filename: str, processor: FreeWiliProcessorType) -> Result[str, str]:
+        """Remove a directory or file on the FreeWili.
+
+        Arguments:
+        ----------
+            dir_or_filename: str
+                The directory or file to remove.
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        ---------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.remove_directory_or_file(dir_or_filename)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def create_blank_file(self, name: str, processor: FreeWiliProcessorType) -> Result[str, str]:
+        """Create a blank file on the FreeWili.
+
+        Arguments:
+        ----------
+            name: str
+                The name of the file to create.
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        ---------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.create_blank_file(name)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def move_directory_or_file(
+        self, original_name: str, new_name: str, processor: FreeWiliProcessorType
+    ) -> Result[str, str]:
+        """Move a directory or file on the FreeWili.
+
+        Arguments:
+        ----------
+            original_name: str
+                The original name of the directory or file to move.
+            new_name: str
+                The new name of the directory or file.
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        ---------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.move_directory_or_file(original_name, new_name)
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def format_filesystem(self, processor: FreeWiliProcessorType) -> Result[str, str]:
+        """Format the filesystem on the FreeWili.
+
+        Arguments:
+        ----------
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        ---------
+            Result[str, str]:
+                Ok(str) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.format_filesystem()
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
+    def list_current_directory(self, processor: FreeWiliProcessorType) -> Result[FileSystemContents, str]:
+        """List the contents of the current directory on the FreeWili.
+
+        Note: This API is currently considered experimental and may change in the future.
+
+        Arguments:
+        ----------
+            processor: FreeWiliProcessorType
+                Processor to use.
+
+        Returns:
+        -------
+            Result[FileSystemContents, str]:
+                Ok(FileSystemContents) if the command was sent successfully, Err(str) if not.
+        """
+        match self.get_serial_from(processor):
+            case Ok(serial):
+                return serial.list_current_directory()
+            case Err(msg):
+                return Err(msg)
+            case _:
+                raise RuntimeError("Missing case statement")
+
     def reset_to_uf2_bootloader(self, processor: FreeWiliProcessorType) -> Result[None, str]:
         """Reset the FreeWili to the uf2 bootloader.
 
