@@ -4,6 +4,7 @@ import queue
 import threading
 import time
 from queue import Queue
+from threading import Lock
 from typing import Any
 
 from result import Err, Ok, Result
@@ -13,6 +14,88 @@ from freewili.framing import ResponseFrame
 from freewili.util.fifo import SafeIOFIFOBuffer
 
 
+class SafeDict:  # noqa: D101
+    """A thread-safe dictionary implementation.
+
+    This class provides a dictionary-like interface with thread safety
+    using a lock to protect all dictionary operations.
+    """
+
+    def __init__(self) -> None:
+        self._lock = Lock()
+        self._dict: dict[Any, Any] = {}
+
+    def __len__(self) -> int:
+        with self._lock:
+            return len(self._dict)
+
+    def __iter__(self) -> Any:
+        with self._lock:
+            return iter(dict(self._dict))
+
+    def __contains__(self, key: Any) -> bool:
+        with self._lock:
+            return key in self._dict
+
+    def get(self, key: Any, default: Any = None) -> Any:  # noqa: D102
+        with self._lock:
+            return self._dict.get(key, default)
+
+    def items(self) -> list[tuple[Any, Any]]:  # noqa: D102
+        with self._lock:
+            return list(self._dict.items())
+
+    def values(self) -> list[Any]:  # noqa: D102
+        with self._lock:
+            return list(self._dict.values())
+
+    def keys(self) -> list[Any]:  # noqa: D102
+        with self._lock:
+            return list(self._dict.keys())
+
+    def clear(self) -> None:  # noqa: D102
+        with self._lock:
+            self._dict.clear()
+
+    def update(self, *args: Any, **kwargs: Any) -> None:  # noqa: D102
+        with self._lock:
+            self._dict.update(*args, **kwargs)
+
+    def setdefault(self, key: Any, default: Any = None) -> Any:  # noqa: D102
+        with self._lock:
+            return self._dict.setdefault(key, default)
+
+    def __getitem__(self, key: Any) -> Any:
+        with self._lock:
+            return self._dict[key]
+
+    def __setitem__(self, key: Any, value: Any) -> None:
+        with self._lock:
+            self._dict[key] = value
+
+    def __delitem__(self, key: Any) -> None:
+        with self._lock:
+            del self._dict[key]
+
+    def pop(self, key: Any) -> Any:  # noqa: D102
+        """Remove and return the value for the given key if it exists."""
+        with self._lock:
+            return self._dict.pop(key, None)
+
+
+class SafeResponseFrameDict(SafeDict):
+    """A thread-safe dictionary for response frames."""
+
+    def __init__(self) -> None:
+        super().__init__()
+
+    def add(self, rf: ResponseFrame) -> None:
+        """Add a ResponseFrame to the container."""
+        """Add an item to the queue."""
+        assert isinstance(rf, ResponseFrame), "Expected a ResponseFrame instance"
+        self.setdefault(rf.rf_type_data, []).append(rf)
+
+
 class SerialPort(threading.Thread):
     """Read/Write data to a serial port."""
 
@@ -33,6 +116,7 @@ class SerialPort(threading.Thread):
         # Response frame queue
         self.rf_queue: Queue = Queue()
         self.rf_event_queue: Queue = Queue()
+        self.rf_events: SafeResponseFrameDict = SafeResponseFrameDict()
         # data other than a response frame
         self.data_queue: Queue = Queue()
 
@@ -49,14 +133,14 @@ class SerialPort(threading.Thread):
         See also: is_open()
 
         Parameters:
-        ----------
+        -----------
             block: bool:
                 If True, block until the serial port is opened.
             timeout_sec: float:
                 number of seconds to wait when blocking.
 
         Returns:
-        -------
+        --------
             None
 
         Raises:
@@ -87,14 +171,14 @@ class SerialPort(threading.Thread):
         See also: is_open()
 
         Parameters:
-        ----------
+        -----------
             block: bool:
                 If True, block until the serial port is closed.
             timeout_sec: float:
                 number of seconds to wait when blocking.
 
         Returns:
-        -------
+        --------
             None
 
         Raises:
@@ -118,11 +202,11 @@ class SerialPort(threading.Thread):
         """Return if the serial port is open.
 
         Parameters:
-        ----------
+        -----------
             None
 
         Returns:
-        -------
+        --------
             bool:
                 True if open, False if closed.
         """
@@ -134,11 +218,11 @@ class SerialPort(threading.Thread):
         To clear the error state, call get_error().
 
         Parameters:
-        ----------
+        -----------
             None
 
         Returns:
-        -------
+        --------
             bool:
                 True if there are errors, False otherwise.
         """
@@ -148,11 +232,11 @@ class SerialPort(threading.Thread):
         """Get the serial port error message. Clears the error state.
 
         Parameters:
-        ----------
+        -----------
             None
 
         Returns:
-        -------
+        --------
             str:
                 Error message if present, empty str otherwise.
         """
@@ -167,7 +251,7 @@ class SerialPort(threading.Thread):
         """Get the serial port descriptor.
 
         Returns:
-        -------
+        --------
             str:
                 serial port descriptor.
         """
@@ -178,7 +262,7 @@ class SerialPort(threading.Thread):
         """Get the serial port baudrate.
 
         Returns:
-        -------
+        --------
             str:
                 serial port baudrate.
         """
@@ -294,7 +378,10 @@ class SerialPort(threading.Thread):
         while frame := data_buffer.pop_first_match(rb"\[\*.*.\d\]\r?\n"):
             self._debug_print(f"RX Event Frame: {frame!r}")
             # self._debug_print(f"Buffer len: {data_buffer.available()} {data_buffer.peek()!r}")
-            self.rf_event_queue.put(ResponseFrame.from_raw(frame))
+            rf_result = ResponseFrame.from_raw(frame)
+            if rf_result.is_ok():
+                self.rf_events.add(rf_result.unwrap())
+            self.rf_event_queue.put(rf_result)
             self._debug_count = 0
         # Match a full response frame
         while frame := data_buffer.pop_first_match(rb"\[[^\*].*.\d\]\r?\n"):
@@ -346,7 +433,7 @@ class SerialPort(threading.Thread):
         r"""Send data to the serial port.
 
         Parameters:
-        ----------
+        -----------
             data : bytes | str:
                 data to be sent to the serial port. If type is str it will be automatically encoded.
             append_newline : bool:
@@ -357,7 +444,7 @@ class SerialPort(threading.Thread):
                 Number of seconds to wait after sending.
 
         Returns:
-        -------
+        --------
             None
         """
         assert isinstance(data, (bytes, str))
